#!/usr/bin/php -Cq
<?php

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

$root_path = dirname($argv[0]);
require_once($root_path.'/config.php');

$silent = true;
$lck_file = $root_path.'/log/rec_tv.lck';
$lck_fp = fopen($lck_file, 'wb');
if (!$lck_fp) {
    writelog("Can't open file: $lck_file");
    exit;
}
if (!flock($lck_fp, LOCK_EX | LOCK_NB)) {
    fclose($lck_fp);
    if (!$silent)
        writelog("Can't lock file: $lck_file");
    exit;
}

$now = strftime('%Y/%m/%d %H:%M');
$curRec = false;
$nextRec = false;
$path = $root_path.'/schedule/';
foreach (glob($path.'*.tv') as $file) {
    $fp = @fopen($file, 'rt');
    if (!$fp) continue;
    $buf = fread($fp, filesize($file));
    fclose($fp);
    $aData = unserialize($buf);
    if (!is_array($aData)) {
        unlink($file);
        writelog("$file format error, deleted");
        continue;
    }
    if ($aData['start_time'] <= $now) {
        if ($aData['end_time'] >= $now) {
            // we should record this now
            if ($curRec !== false) {
                if ($aData['start_time'] > $curRec['start_time']) {
                    $pre_file = $path.$curRec['fname'].'.tv';
                    unlink($pre_file);
                    writelog($curRec['fname']." expired, replace by ".$aData['fname']);
                    $curRec = $aData;
                }
                else {
                    writelog($aData['fname']." expired, already has ".$curRec['fname']);
                }
            }
            else
                $curRec = $aData;
        }
        else {
            // expire
            unlink($file);
            writelog($aData['fname']." expired, now = $now, end_time = ".$aData['end_time']);
        }
    }
    else {
        if ($nextRec === false || $aData['start_time'] < $nextRec['start_time'])
            $nextRec = $aData;
    }
}

if ($curRec != false) {
    $lck_ctlfile = $root_path.'/log/tvctrl.lck';
    $lck_ctlfp = fopen($lck_ctlfile, 'wb');
    if ($lck_ctlfp) {
        if (!flock($lck_ctlfp, LOCK_EX)) {
            fclose($lck_ctlfp);
            writelog("Can't lock file: $lck_ctlfile");
        }
        else {
            $shcmd = $root_path.'/scripts/check.sh';
            $pipe = popen($shcmd, 'r');
            if (!$pipe) {
                writelog("popen($shcmd) failed!");
            }
            else {
                $pid = trim(fread($pipe, 1024));
                pclose($pipe);
                if ($pid === '' || $pid == 0) {
                    writelog("no any mytv server");
                }
                else {
                    $sig = 9;
                    if (!posix_kill($pid, $sig))
                        writelog("Failed to kill mytv process $pid!");
                    else {
                        for ($i = 0; $i < $max_delay; $i++) {
                            sleep(1);
                            if (!posix_kill($pid, 0)) break;
                        }
                        writelog("mytv process $pid killed!");
                    }
                }
            }
            $shcmd = $root_path.'/scripts/rec_check.sh';
            $pipe = popen($shcmd, 'r');
            if (!$pipe) {
                writelog("popen($shcmd) failed!");
            }
            else {
                $pid = trim(fread($pipe, 1024));
                pclose($pipe);
                if ($pid === '' || $pid == 0) {
                    writelog("no any recorder");
                }
                else {
                    $sig = 9;
                    if (!posix_kill($pid, $sig))
                        writelog("Failed to kill recorder process $pid!");
                    else {
                        for ($i = 0; $i < $max_delay; $i++) {
                            sleep(1);
                            if (!posix_kill($pid, 0)) break;
                        }
                        writelog("recorder process $pid killed!");
                    }
                }
            }
            $len = getTimeLength($now, $curRec['end_time']);
            $channel = $curRec['channel'];
            $width = $curRec['width'];
            $height = $curRec['height'];
            $file = $root_path.'/tv/'.$recorder_fname_start.strftime('%Y%m%d-%H%M').'-'.$channel.'-'.$width.'x'.$height.'-'.$len.$recorder_fname_extension;
            $shcmd = $root_path.'/scripts/rec_tv.sh '.$file.' '.$channel.' '.$len.' '.$width.' '.$height;
            writelog("execute: $shcmd");
            flock($lck_ctlfp, LOCK_UN);
            fclose($lck_ctlfp);
            $pipe = popen($shcmd, 'r');
            if (!$pipe)
                writelog("- popen($shcmd) failed!");
            else
                pclose($pipe);
            $pre_file = $path.$curRec['fname'].'.tv';
            unlink($pre_file);
            writelog($curRec['fname']." recorded to $file");
        }
    }
    else {
        writelog("Can't open file: $lck_ctlfile");
    }
}
else if (!$silent) {
    if ($nextRec === false)
        writelog("no any scheduled task");
    else
        writelog("next task will be ".$nextRec['fname']." at ".$nextRec['start_time']." to ".$nextRec['end_time']);
}

flock($lck_fp, LOCK_UN);
fclose($lck_fp);
exit;

function getTimeLength($start_time, $end_time)
{
    $s_year = substr($start_time, 0, 4);
    $s_month = substr($start_time, 5, 2);
    $s_day = substr($start_time, 8, 2);
    $s_hour = substr($start_time, 11, 2);
    $s_minute = substr($start_time, 14, 2);
    $e_year = substr($end_time, 0, 4);
    $e_month = substr($end_time, 5, 2);
    $e_day = substr($end_time, 8, 2);
    $e_hour = substr($end_time, 11, 2);
    $e_minute = substr($end_time, 14, 2);
    $stime = mktime($s_hour, $s_minute, 0, $s_month, $s_day, $s_year);
    $etime = mktime($e_hour, $e_minute, 59, $e_month, $e_day, $e_year);
    $length = $etime - $stime + 1;
    if ($length < 1)
        return 1;
    if ($length > 21600)
        return 21600;
    return $length;
}

function writelog($buf)
{
    global $root_path;

    $file = $root_path.'/log/rec_tv_'.strftime('%Y%m%d').'.log';
    $fp = @fopen($file, 'at');
    if ($fp != 0) {
        $msg = sprintf('%s [%d] %s', strftime('%T'), getmypid(), $buf);
        fputs($fp, $msg."\n");
        fclose($fp);
    }
    return;
}

?>
