#!/usr/bin/php -Cq
<?php

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

$root_path = dirname($argv[0]);
require_once($root_path.'/config.php');

$fp = fopen('php://stdin', 'rt');
if ($fp == 0) {
    writelog('Failed to open php://stdin!');
    exit;
}

echo "+ OK\n";

while (!feof($fp)) {
    $buf = trim(fgets($fp, 4096));
    if (feof($fp)) break;
    $aCmd = explode(' ', $buf);
    if ($aCmd === false) {
        writelog(">> $buf");
        continue;
    }
    $cmd = $aCmd[0];
    if (strcasecmp($cmd, 'QUIT') == 0) {
        writelog(">> $buf");
        $rbuf = '+ QUIT!';
        echo "$rbuf\n";
        writelog("<< $rbuf");
        break;
    }
    if (strcasecmp($cmd, 'REC_GET_FILE') == 0) {
        $fileid = trim($aCmd[1]);
        $file = '';
        $aRecFile = getAllRecFile();
        if ($aRecFile !== false && is_array($aRecFile)) {
            foreach ($aRecFile as $aData) {
                if (!is_array($aData)) continue;
                $id = md5($aData['file']);
                if ($id === $fileid) {
                    $file = $aData['file'];
                    break;
                }
            }
        }
        if ($file === '')
            $rbuf = "- no data";
        else
            $rbuf = "+ ".base64_encode($file);
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'LOGIN') == 0) {
        $user = trim(base64_decode($aCmd[1]));
        $pass = base64_decode($aCmd[2]);
        writelog("[$user] >> $buf");
        if ($using_smtp_auth)
            $auth_result = CheckAuthSMTP($user, $pass);
        else
            $auth_result = CheckAuthPOP($user, $pass);
        if (!$auth_result) {
            $rbuf = '- ERR';
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        unset($aSession);
        writelog("[$user] User login!");
        $bAllowRec = CheckRecUsers($user);
        if ($bAllowRec)
            writelog("User $user can access recorder feature!");
        $aSession['tm'] = time();
        $aSession['user'] = $user;
        $aSession['allow_rec'] = $bAllowRec;
        $sid = saveSessionData($aSession);
        if ($sid === false)
            $rbuf = '- Failed to save session data!';
        else {
            if ($bAllowRec)
                $rbuf = "+ $sid 1";
            else
                $rbuf = "+ $sid 0";
        }
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    $sid = $aCmd[1];
    unset($aSession);
    $aSession = getSessionData($sid);
    if ($aSession === false) {
        writelog(">> $buf");
        $rbuf = "! No session for $sid, please LOGIN first!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    $user = $aSession['user'];
    writelog("[$user] >> $buf");
    $now = time();
    if ($now >= $aSession['tm'])
        $len = $now - $aSession['tm'];
    else
        $len = $now;
    if ($len > $session_timeout) {
        deleteSession($sid);
        $rbuf = "! Session timeout for $user/$sid, please LOGIN again!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    $aSession['tm'] = time();
    saveSessionData($aSession, $sid);
    if (strcasecmp($cmd, 'LOGOUT') == 0) {
        deleteSession($sid);
        $rbuf = "+ LOGOUT for $user/$sid!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'SESSION') == 0) {
        if ($aSession['allow_rec'])
            $rbuf = "+ $user 1";
        else
            $rbuf = "+ $user 0";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'STATUS') == 0) {
        $shcmd = $root_path.'/scripts/check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        $aServerData = getServerData();
        $bandwidth = base64_encode($aServerData['bandwidth']);
        $size = base64_encode($aServerData['size']);
        $aRecorderData = getRecorderData();
        $recorder_size = base64_encode($aRecorderData['size']);
        $ch = getChannelData();
        $channel = base64_encode($ch);
        if ($server_file === '')
            $fname = base64_encode($aServerData['filename']);
        else
            $fname = base64_encode($server_file);
        $vol = getVolumeData();
        $volume = base64_encode($vol);
        if ($pid !== '')
            $rbuf = "+ $pid $bandwidth $size $channel $fname $volume $recorder_size";
        else
            $rbuf = "+ - $bandwidth $size $channel $fname $volume $recorder_size";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_STATUS') == 0) {
        $shcmd = $root_path.'/scripts/rec_check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid === '')
            $pid = '-';
        $aRecData = getNextRecData();
        if ($aRecData !== false && is_array($aRecData)) {
            $fname = base64_encode($aRecData['fname']);
            $start_time = base64_encode($aRecData['start_time']);
            $end_time = base64_encode($aRecData['end_time']);
            $channel = base64_encode($aRecData['channel']);
        }
        else {
            $fname = base64_encode('-');
            $start_time = $fname;
            $end_time = $fname;
            $channel = $fname;
        }
        $rbuf = "+ $pid $start_time $end_time $channel $fname";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_LIST') == 0) {
        $aRecData = getAllRecData();
        if ($aRecData !== false && is_array($aRecData)) {
            foreach ($aRecData as $aData) {
                if (!is_array($aData)) continue;
                $fname = base64_encode($aData['fname']);
                $start_time = base64_encode($aData['start_time']);
                $end_time = base64_encode($aData['end_time']);
                $channel = base64_encode($aData['channel']);
                $width = base64_encode($aData['width']);
                $height = base64_encode($aData['height']);
                $rbuf = "+ $start_time $end_time $channel $width $height $fname";
                echo "$rbuf\n";
                writelog("[$user] << $rbuf");
            }
        }
        $rbuf = "- no data";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_LIST_FILE') == 0) {
        $aRecFile = getAllRecFile();
        if ($aRecFile !== false && is_array($aRecFile)) {
            foreach ($aRecFile as $aData) {
                if (!is_array($aData)) continue;
                $file = base64_encode($aData['file']);
                $size = base64_encode($aData['size']);
                $rbuf = "+ $file $size";
                echo "$rbuf\n";
                writelog("[$user] << $rbuf");
            }
        }
        $rbuf = "- no data";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_DEL_FILE') == 0) {
        $file = trim(base64_decode($aCmd[2]));
        $fname = $root_path.'/tv/'.$file;
        if (file_exists($fname))
            unlink($fname);
        $rbuf = "+ deleted file $file";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_ADD') == 0) {
        $start_time = trim(base64_decode($aCmd[2]));
        $end_time = trim(base64_decode($aCmd[3]));
        $channel = base64_decode($aCmd[4]);
        $recorder_size = base64_decode($aCmd[5]);
        if (!is_numeric($channel) || $channel < 1 || $channel > 128) {
            $rbuf = "- Channel should be a number and between 1 and 127";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $now = strftime('%Y/%m/%d %H:%M');
        if (!is_string($start_time) || !preg_match('/[0-9][0-9][0-9][0-9]\/[0-1][0-9]\/[0-3][0-9] [0-2][0-9]:[0-5][0-9]/', $start_time)) {
            $rbuf = "- Start Time format should be YYYY/MM/DD HH:MM";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (!is_string($end_time) || !preg_match('/[0-9][0-9][0-9][0-9]\/[0-1][0-9]\/[0-3][0-9] [0-2][0-9]:[0-5][0-9]/', $end_time)) {
            $rbuf = "- End Time format should be YYYY/MM/DD HH:MM";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if ($start_time < $now) {
            $rbuf = "- Start Time ($start_time) < Now ($now)";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if ($end_time < $start_time) {
            $rbuf = "- End Time ($end_time) < Start Time ($start_time)";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $length = getTimeLength($start_time, $end_time);
        if ($length < 0 || $length > 21600) {
            $rbuf = "- Max Record Time is 6 hours (21600 seconds), this is $length seconds";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (checkDuplicateRec($start_time, $end_time)) {
            $rbuf = "- Time overlap with some scheduled task";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        saveRecorderData(array('size' => $recorder_size));
        addRecData($sid, $user, $start_time, $end_time, $channel, $recorder_size);
        $rbuf = "+ added";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_DEL') == 0) {
        $fname = base64_decode($aCmd[2]);
        deleteRecData($fname);
        $rbuf = "+ deleted";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'REC_STOP') == 0) {
        $shcmd = $root_path.'/scripts/rec_check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid === '') {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if ($pid == 0) {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (isset($aCmd[2]) && strcasecmp($aCmd[2], 'FORCE') == 0)
            $sig = 9;
        else
            $sig = 15;
        if (!posix_kill($pid, $sig)) {
            $rbuf = "- Failed to kill process!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        for ($i = 0; $i < $max_delay; $i++) {
            sleep(1);
            if (!posix_kill($pid, 0)) break;
        }
        $rbuf = "+ Process killed! If still exist, please try STOP FORCE!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'START') == 0) {
        $lck_file = $root_path.'/log/tvctrl.lck';
        $fp = fopen($lck_file, 'wb');
        if (!$fp) {
            $rbuf = "- fopen($lck_file) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (!flock($fp, LOCK_EX | LOCK_NB)) {
            fclose($fp);
            $rbuf = "- flock($lck_file) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $shcmd = $root_path.'/scripts/check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '') {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- Process exist: $pid!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        // wait 2 seconds
        sleep(2);
        $shcmd = $root_path.'/scripts/rec_check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '') {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- Recorder Process exist: $pid!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (!isset($aCmd[3])) {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- Syntax: START sid bandwidth screensize";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $bandwidth = base64_decode($aCmd[2]);
        $size = base64_decode($aCmd[3]);
        $channel = getChannelData();
        $volume = getVolumeData();
        if ($server_file === '')
            $fname = md5(strftime('%Y%m%d%H%M%S').rand(1, 256)).'.rm';
        else
            $fname = $server_file;
        $shcmd = $root_path.'/scripts/start.sh "'.$bandwidth.'" "'.$size.'" "'.$channel.'" "'.$fname.'" "'.$volume.'"';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        pclose($pipe);
        $shcmd = $root_path.'/scripts/check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        flock($fp, LOCK_UN);
        fclose($fp);
        if ($pid !== '')
            $rbuf = "+ Process started: $pid!";
        else
            $rbuf = "- Failed to start the process!";
        echo "$rbuf\n";
        saveServerData(array('bandwidth' => $bandwidth, 'size' => $size, 'filename' => $fname));
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'STOP') == 0) {
        $shcmd = $root_path.'/scripts/check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid === '') {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if ($pid == 0) {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        if (isset($aCmd[2]) && strcasecmp($aCmd[2], 'FORCE') == 0)
            $sig = 9;
        else
            $sig = 15;
        if (!posix_kill($pid, $sig)) {
            $rbuf = "- Failed to kill process!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        for ($i = 0; $i < $max_delay; $i++) {
            sleep(1);
            if (!posix_kill($pid, 0)) break;
        }
        $rbuf = "+ Process killed! If still exist, please try STOP FORCE!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'CHANNEL') == 0) {
        if (!isset($aCmd[2])) {
            $rbuf = "- Syntax: CHANNEL channel";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $channel = base64_decode($aCmd[2]);
        if (!is_numeric($channel) || $channel < 1 || $channel > 128) {
            $rbuf = "- Channel should be a number and between 1 and 127";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $shcmd = $root_path.'/scripts/rec_check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '') {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- Recorder Process exist: $pid!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $shcmd = $root_path.'/scripts/channel.sh '.$channel;
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        pclose($pipe);
        saveChannelData($channel);
        $rbuf = "+ Channel changed, please wait!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'VOLUME') == 0) {
        if (!isset($aCmd[2])) {
            $rbuf = "- Syntax: VOLUME vol";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $vol = base64_decode($aCmd[2]);
        if (!is_numeric($vol) || $vol < 0 || $vol > 100) {
            $rbuf = "- Volume should be a number and between 0 and 100";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $shcmd = $root_path.'/scripts/rec_check.sh';
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '') {
            flock($fp, LOCK_UN);
            fclose($fp);
            $rbuf = "- Recorder Process exist: $pid!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        $shcmd = $root_path.'/scripts/volume.sh '.$vol;
        writelog("[$user] execute: $shcmd");
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("[$user] << $rbuf");
            continue;
        }
        pclose($pipe);
        saveVolumeData($vol);
        $rbuf = "+ Volume changed, please wait!";
        echo "$rbuf\n";
        writelog("[$user] << $rbuf");
        continue;
    }

    $rbuf = "- Unknown command: $cmd";
    echo "$rbuf\n";
    writelog("[$user] << $rbuf");
}
fclose($fp);

exit;

function CheckAuthSMTP($username, $password)
{
    global $aUsers;

    if (!in_array($username, $aUsers)) return false;
    $fp = @fsockopen('localhost', 25);
    if (!$fp) return false;
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '220', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, "EHLO localhost\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '250', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, "AUTH LOGIN\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '334', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, base64_encode($username)."\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '334', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, base64_encode($password)."\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    fputs($fp, 'QUIT\n');
    fclose($fp);
    if (strncmp($buf, '235', 3))
        return false;
    return true;
}

function CheckAuthPOP($username, $password)
{
    global $aUsers;

    if (!in_array($username, $aUsers)) return false;
    $fp = @fsockopen('localhost', 110);
    if (!$fp) return false;
    $buf = fgets($fp);
    if ($buf[0] != '+') {
        fclose($fp);
        return false;
    }
    fputs($fp, "USER $username\n");
    $buf = fgets($fp);
    if ($buf[0] != '+') {
        fclose($fp);
        return false;
    }
    fputs($fp, "PASS $password\n");
    $buf = fgets($fp);
    fclose($fp);
    if ($buf[0] != '+')
        return false;
    return true;
}

function CheckRecUsers($username)
{
    global $aRecUsers;

    if (!isset($aRecUsers) || !is_array($aRecUsers)) return false;
    if (!in_array($username, $aRecUsers)) return false;
    return true;
}

function deleteSession($sid)
{
    global $root_path;

    $sid_file = $root_path.'/session/'.$sid.'.session';
    unlink($sid_file);
    return;
}

function getSessionData($sid)
{
    global $root_path;

    if ($sid === '' || $sid === false) {
        writelog("No SID!");
        return false;
    }
    $sid_file = $root_path.'/session/'.$sid.'.session';
    if (!is_file($sid_file)) {
        writelog("SID $sid not exist!");
        return false;
    }
    $fp = @fopen($sid_file, 'rt');
    if (!$fp) {
        writelog("Can't open SID file: $sid_file!");
        return false;
    }
    $buf = fread($fp, filesize($sid_file));
    fclose($fp);
    return unserialize($buf);
}

function saveSessionData($aSession, $mysid = false)
{
    global $root_path;

    $s = serialize($aSession);
    if ($mysid === false)
        $sid = md5($s);
    else
        $sid = $mysid;
    $sid_file = $root_path.'/session/'.$sid.'.session';
    $fp = @fopen($sid_file, 'w+t');
    if (!$fp) {
        writelog("Can't open SID file: $sid_file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return $sid;
}

function getRecorderData()
{
    global $root_path, $default_recorder_size;

    $file = $root_path.'/session/recorder.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        $aData = array('size' => $default_recorder_size);
        saveRecorderData($aData);
        return $aData;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveRecorderData($aData)
{
    global $root_path;

    $s = serialize($aData);
    $file = $root_path.'/session/recorder.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Recorder file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function getServerData()
{
    global $root_path, $default_bandwidth, $default_size;

    $file = $root_path.'/session/server.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        $aData = array('bandwidth' => $default_bandwidth, 'size' => $default_size);
        saveServerData($aData);
        return $aData;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveServerData($aData)
{
    global $root_path;

    $s = serialize($aData);
    $file = $root_path.'/session/server.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Server file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function getChannelData()
{
    global $root_path, $default_channel;

    $file = $root_path.'/session/channel.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        saveChannelData($default_channel);
        return $default_channel;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveChannelData($ch)
{
    global $root_path;

    $s = serialize($ch);
    $file = $root_path.'/session/channel.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Channel file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function getVolumeData()
{
    global $root_path, $default_volume;

    $file = $root_path.'/session/volume.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        saveVolumeData($default_volume);
        return $default_volume;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveVolumeData($vol)
{
    global $root_path;

    $s = serialize($vol);
    $file = $root_path.'/session/volume.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Volume file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function addRecData($sid, $user, $start_time, $end_time, $channel, $recorder_size)
{
    global $root_path;

    list($width, $height) = explode('x', $recorder_size);
    $fname = md5($sid.'_'.time().'_'.$user.'_'.$start_time.'_'.$end_time.'_'.$channel.'_'.$width.'x'.$height);
    $file = $root_path.'/schedule/'.$fname.'.tv';
    $s = serialize(array(
            'fname' => $fname,
            'start_time' => $start_time,
            'end_time' => $end_time,
            'channel' => $channel,
            'height' => $height,
            'width' => $width,
            'user' => $user,
            ));
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open schedule file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function deleteRecData($file)
{
    global $root_path;

    $path = $root_path.'/schedule/';
    $fname = $path.$file.'.tv';
    if (file_exists($fname))
        unlink($fname);
    return;
}

function checkDuplicateRec($start_time, $end_time)
{
    global $root_path;

    $now = strftime('%Y/%m/%d %H:%M');
    $path = $root_path.'/schedule/';
    foreach (glob($path.'*.tv') as $file) {
        $fp = @fopen($file, 'rt');
        if (!$fp) continue;
        $buf = fread($fp, filesize($file));
        fclose($fp);
        $aData = unserialize($buf);
        if (!is_array($aData)) continue;
        if ($start_time >= $aData['start_time']) {
            if ($start_time <= $aData['end_time'])
                return true;
        }
        else if ($end_time >= $aData['start_time'])
            return true;
    }
    return false;
}

function getAllRecData()
{
    global $root_path;

    $aRecData = array();
    $now = strftime('%Y/%m/%d %H:%M');
    $path = $root_path.'/schedule/';
    foreach (glob($path.'*.tv') as $file) {
        $fp = @fopen($file, 'rt');
        if (!$fp) continue;
        $buf = fread($fp, filesize($file));
        fclose($fp);
        $aData = unserialize($buf);
        if (!is_array($aData)) continue;
        if ($aData['start_time'] > $now) {
            $aRecData[$aData['start_time']] = $aData;
        }
    }
    ksort($aRecData);
    return $aRecData;
}

function getNextRecData()
{
    global $root_path;

    $now = strftime('%Y/%m/%d %H:%M');
    $nextRec = false;
    $curRec = false;
    $path = $root_path.'/schedule/';
    foreach (glob($path.'*.tv') as $file) {
        $fp = @fopen($file, 'rt');
        if (!$fp) continue;
        $buf = fread($fp, filesize($file));
        fclose($fp);
        $aData = unserialize($buf);
        if (!is_array($aData)) continue;
        if ($aData['start_time'] <= $now && $aData['end_time'] >= $now) {
            // recording
            $curRec = $aData;
        }
        else if ($aData['start_time'] >= $now) {
            if ($nextRec === false || $nextRec['start_time'] > $aData['start_time'])
                $nextRec = $aData;
        }
    }
    if ($curRec !== false)
        return $curRec;
    return $nextRec;
}

function getAllRecFile()
{
    global $root_path, $recorder_fname_extension;

    $aRecFile = array();
    $path = $root_path.'/tv/';
    foreach (glob($path.'*'.$recorder_fname_extension) as $file) {
        $fp = @fopen($file, 'rt');
        if (!$fp) continue;
        $size = filesize($file);
        fclose($fp);
        $aRecFile[] = array('file' => $file, 'size' => $size);
    }
    return $aRecFile;
}

function getTimeLength($start_time, $end_time)
{
    $s_year = substr($start_time, 0, 4);
    $s_month = substr($start_time, 5, 2);
    $s_day = substr($start_time, 8, 2);
    $s_hour = substr($start_time, 11, 2);
    $s_minute = substr($start_time, 14, 2);
    $e_year = substr($end_time, 0, 4);
    $e_month = substr($end_time, 5, 2);
    $e_day = substr($end_time, 8, 2);
    $e_hour = substr($end_time, 11, 2);
    $e_minute = substr($end_time, 14, 2);
    $stime = mktime($s_hour, $s_minute, 0, $s_month, $s_day, $s_year);
    $etime = mktime($e_hour, $e_minute, 59, $e_month, $e_day, $e_year);
    $length = $etime - $stime + 1;
    return $length;
}

function writelog($buf)
{
    global $root_path;

    $file = $root_path.'/log/'.strftime('%Y%m%d').'.log';
    $fp = @fopen($file, 'at');
    if ($fp != 0) {
        $msg = sprintf('%s [%d] %s', strftime('%T'), getmypid(), $buf);
        fputs($fp, $msg."\n");
        fclose($fp);
    }
    return;
}

?>
